#!/usr/bin/perl -w

# This nagios plugin come with ABSOLUTELY NO WARRANTY. You may redistribute copies
# of the plugins under the terms of the GNU General Public License. 
# For more information about these matters, see the file named COPYING.
#
# revision 0.2 (i.e. Extreme Beta use at your own risk)
#

use strict;

use lib "/usr/local/nagios/libexec";
use utils qw($TIMEOUT %ERRORS &print_revision &support);
use vars qw($PROGNAME);
my ($opt_V,$opt_h, $opt_c, $opt_t, $result, $state, $text_marker, $output, $l1, $l2, $len_written, $reached_eof, $file_number );
my ($temp_time, $oldest_file, $BUFFER_SIZE);

$BUFFER_SIZE = 1024;
$state = 'OK';
$len_written = 0;
$output = '';
$text_marker = 0;

$ENV{'BASH_ENV'}='';
$ENV{'ENV'}='';
$ENV{'PATH'}='';

$PROGNAME = "get_info";

sub print_help ();
sub print_usage ();
sub get_info();
sub get_text();
sub print_output();


use Getopt::Long;
Getopt::Long::Configure('bundling');
GetOptions(
       	"V"   => \$opt_V, "version"    => \$opt_V,
        "h"   => \$opt_h, "help"       => \$opt_h,
	"c=s" => \$opt_c, "command=s"  => \$opt_c,
	"t=s" => \$opt_t, "text_file=s"  => \$opt_t,
           );

# -h means display verbose help screen
if ($opt_h) { print_help(); exit $ERRORS{'OK'}; }

# -V means display version number
if ($opt_V) {
        print_revision($PROGNAME,'$Revision: 0.2 $ ');
        exit $ERRORS{'OK'};
}

# -c means host name
unless ($opt_c or $opt_t) { print_usage(); exit $ERRORS{'UNKNOWN'}; }


# Just in case of problems, let's not hang Nagios
$SIG{'ALRM'} = sub {
        print ("ERROR: No response from nrpe (alarm)\n");
        exit $ERRORS{"UNKNOWN"};
};
alarm($TIMEOUT);


if($opt_c){
	&get_info();
}
elsif($opt_t) {
	&get_text();
}
else{
	$state = 'UNKNOWN';
}

exit $ERRORS{$state};




########  Subroutines ==========================

sub get_info() {
	

	$result = `$opt_c`;

	if( length($result) + 3 + 1 <= $BUFFER_SIZE){
		$result =~ s/\n/~/g;
		$result = $result . "EOF";
		print $result;
		return;
	}
	else {
		# find a file to write to
		$file_number = 1;
		while($file_number < 10){
			if( -e "/usr/local/nagios/text$file_number"){
				$file_number++;
				next;
                	}	
			last;
		} # EOW

		if($file_number >= 10){
			#OK all ten are used so let's use the oldest one.  
			$oldest_file = $file_number = 1;
                        $temp_time = (stat("/usr/local/nagios/text$file_number"))[10];
			$file_number++;
			while($file_number < 10){
				if( $temp_time < (stat("/usr/local/nagios/text$file_number"))[10]){
                                	$file_number++;
                                	next;
                        	}
				$temp_time = (stat("/usr/local/nagios/text$file_number"))[10];
				$oldest_file = $file_number;
				$file_number++;
                        	next;
                	} # EOW
			$file_number = $oldest_file;

		}

		# open the text file for writing
		if (!open(FILE, ">/usr/local/nagios/text$file_number")) {
                        print "Can't open the text file $!\n";
                        exit $ERRORS{'CRITICAL'};
		}

# testing
#print "/usr/local/nagios/text$file_number\n";
		
		seek FILE, 0, 0;
		print FILE $result;
		close FILE;

		# open the text file for reading
                if (!open(FILE, "/usr/local/nagios/text$file_number")) {
                        print "Can't open the text file $!\n";
                        exit $ERRORS{'CRITICAL'};
                }

                seek FILE, 0, 0;
		
		&print_output();        

		return;
	
	}

}


sub get_text() {
	
	# test to see if the text marker file exists.
	if(! -e "/usr/local/nagios/$opt_t"."_marker") {
		 print "The marker file doesn't exit $!\n";
                 exit $ERRORS{'CRITICAL'};
	}

	# open the text_marker file for reading  
	if(! open(MARKER, "/usr/local/nagios/$opt_t"."_marker")) {
		print "Can't create the text marker file $!\n";
		exit $ERRORS{'CRITICAL'};
	}
	else {
		# get the first line of the text_marker file, discard the rest of the file
		($text_marker) = <MARKER>;
		close MARKER;
# testing
#print "Text Marker: $text_marker\n";

	}


	if (!open(FILE, "/usr/local/nagios/$opt_t")) {
		print "Can't open the text file $!\n";
		exit $ERRORS{'CRITICAL'};
	}


	seek FILE, $text_marker, 0;	
	$opt_t =~ /([1-9])/;
	$file_number = $1;

# testing
#print "/usr/local/nagios/text$file_number\n";

	&print_output();
	return;

}

sub print_output(){


		$output = '';
		$reached_eof = 1;  	
	
		while(<FILE>){
			chomp($_);
			$l1 = length($output);
			$l2 = length($_);
			if( $l1 + $l2 + 5 + 1 + 1 <= $BUFFER_SIZE){
				$output = $output . $_ . '~';
			}
			else{
				#Since we got here there must be more text in the file, therefore we haven't reached the EOF
				$reached_eof = 0;  
				last
			}
		}	
# testing
#print "Reached EOF: $reached_eof\n";

		if(!$reached_eof){
			$output = $output . "text$file_number";
			$len_written = $text_marker + length($output) - 5;

		        print $output;
			close FILE;

		
			if(! open(MARKER, ">/usr/local/nagios/text$file_number"."_marker")) {
				print "Can't create the text marker file $!\n";
				exit $ERRORS{'CRITICAL'};
			}


			seek MARKER, 0, 0;	
			print MARKER $len_written;
			close MARKER;
		}
		else{    
			#reached eof so print output w/eof and unlink the file
			$output = $output . 'EOF';
			unlink("/usr/local/nagios/text$file_number", "/usr/local/nagios/text$file_number"."_marker");
                        print $output;
                        close FILE;

		}
		return;
}

sub print_help() {
        print_revision($PROGNAME,'$Revision: 0.2 $ ');
        print "Copyright (c) 2002 Daniel Rusch & Daniel Kressin\n";
        print "\n";
        print "\n";
        print "\n";
	print "This plugin is used to return multiple 1k blocks back to Nagios via nrpe\n";
        print "The first time it is executed it takes the name of the command that you wish to execute as the parameter\n";
        print "If there is 1k or less of data the plugin simply prints the data to STDOUT\n";
        print "If there is more than 1k of data the plugin will write the data to disk and then\n";
        print "print the first 1k to STDOUT. On each subsequent call (using the -t option) the plugin \n";
        print "will print the next 1k of data to STDOUT until all data is printed.\n";
        print "\n";
        print "To use this plugin the nrpe command should have the following form:\n";
	print "command[get_ps]=/usr/local/nagios/libexec/get_info -c '/bin/ps -few' \n";
        print "\n";
        print "Additionally, a series of get_text# nrpe commands must be added to the nrpe.cfg file.\n";
        print "command[get_text1]=/usr/local/nagios/libexec/get_info -t text1\n";
	print "command[get_text2]=/usr/local/nagios/libexec/get_info -t text2\n";
	print ".\n";
	print "command[get_text2]=/usr/local/nagios/libexec/get_info -t text9\n";
        print "\n";
        print_usage();
        print "\n";
        support();
}

sub print_usage () {
        print "Usage: \n";
        print " $PROGNAME -c 'command [params]' | -t text# \n";
        print " $PROGNAME [-h | --help]\n";
        print " $PROGNAME [-V | --version]\n";
}

